import { URL } from 'url';
import { join } from 'path';
import crypto from 'crypto';
import axios from 'axios';
import { Logger } from '@stackbit/types';
import { getCommandRunner } from '@stackbit/cms-core';

export type Extension = {
    id: number;
    integrationName: string;
    integrationSlug: string;
    lightLogoPath: string;
    darkLogoPath: string;
    createdAt: string;
    updatedAt: string;
    teamInstallationId: number;
    siteId: string;
    dataLayerId: string;
    configurationType: 'create';
    config: Record<string, string>;
    sourceUrl: string;
    name: string;
    hostSiteUrl: string;
    prefix: string;
};

type GetContentSourceParams = {
    siteId: string;
    teamId: string;
    netlifyAccessToken?: string;
    projectId?: string;
    apiSecret?: string;
    baseURL: string;
};

export const getRemoteContentSources = async ({ siteId, teamId, projectId, netlifyAccessToken, apiSecret, baseURL }: GetContentSourceParams) => {
    try {
        const headers: Record<string, string> = {
            'Content-Type': 'application/json',
            authorization: `Bearer ${netlifyAccessToken ?? apiSecret}`
        };
        let urlPath = 'core/site-extensions-config';

        if (netlifyAccessToken) {
            headers['x-nf-request-id'] = crypto.randomBytes(16).toString('hex');
        } else {
            urlPath = `${urlPath}/${projectId}`;
        }

        const url = new URL(urlPath, baseURL);
        url.searchParams.set('configuration_type', 'create');
        url.searchParams.set('site_id', siteId);
        url.searchParams.set('team_id', teamId);
        const response = await axios<Extension[]>(url.toString(), { headers });
        return response.data;
    } catch (error) {
        return [];
    }
};

export const installRemoteContentSources = async ({
    extensions,
    appDir,
    logger
}: {
    extensions: Extension[];
    appDir: string;
    logger: Logger;
}): Promise<{ path: string; id: number; config: Record<string, unknown> }[]> => {
    const modulesToInstall = extensions.map((extension, index) => `${extension.integrationSlug}-connector@${extension.sourceUrl}?v=${Math.random()}-${index}`);
    const contentSourcesPath = join(appDir, '.stackbit/cache/remote-content-sources');
    const result = extensions.map((extension) => ({
        id: extension.id,
        config: extension.config,
        path: join(contentSourcesPath, `node_modules/${extension.integrationSlug}-connector/netlify-create.js`)
    }));

    const runCommand = getCommandRunner({ env: process.env });
    try {
        await runCommand('npm', ['install', ...modulesToInstall, '--no-save', '--prefix', contentSourcesPath, '--exact'], {
            env: process.env,
            cwd: appDir,
            logger
        });

        return result;
    } catch (error) {
        logger.error(`Error installing remote content sources: ${modulesToInstall.join(',')}`, error);
        return [];
    }
};
